// Import the page-mod API
var pageMod = require("sdk/page-mod");
// Import the self API
var data = require("sdk/self").data;
// import the preferences
var sp = require("sdk/simple-prefs"); // to show the customize button
var tabs = require("sdk/tabs");// to open the options page
var ss = require("sdk/simple-storage"); // to store user specific settings

var DEFAULT_ICON_BASE_64 = "iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAMbElEQVR42tVYC3BU5RU+5967793s5r1JSMIjD16Rd5SXgKAySqGFjkWk" +
			"I4hYlerY6UxnrLWODLa1Hd9QfDC+K1TQyhgriASBBAHlKSGEdwgQQpJNdrOP7N7H3/Pv3lu3lEfCAOKdfLN3s/+993znfOf851yEq3jsXPy0Wbhzpq+9I4AOm/2+E" +
			"cUD/nmln4FXy/gDAyDNkpX9pv/dqik2q02MxWIzyvJ6f/yjIXBoIKy0CDCjraIO7HYnKIoytZ+3sOJHQeBAmXCfyLRljAEEP6tFh8MJsZg8mSKw9ronUNMfikWEHQ" +
			"KAa2sHsqL1e9DtSgGl4fjEQSPGV17XBKpLwOSRYLMIUI5056YYwNGFy6B4eDlkPvublSXL1v/iuiawtx8ukpD9nryPAt1ZFIAdzu2P8qKlULpwPhNOHp3XZ5fy1nV" +
			"JYGd/cazE1EqSj0jGI32CmQioDPDIHXOgcPcGJjXWB+n70JLv4PB1RWBLCbitAuyREAq44RwUBbBIiQd0MvpmMoEoR0FhsK1Tg7GDakC5bghsL4V/mBBmSQnjGY+A" +
			"TBVoW1hgt3k05NWIPM/oE1Var2jwTP8a+MN1QaC6rzjbzNR3yXiUdN0LAuCqTie8DelskdiIo2wx0FSSEz1P04BHQSHcUrYfNv+gBDYWQS/y/C7Sego3XiKliFSCv" +
			"gMbvOIdCorJDK6AH54O7IZcQQOVjNdYnADIGtRTlIYMrYX2H4TA+4WAPUyw2Yow2kReJ+NRIuN9gsBeK52ELK8QJMnEOgIdmHF4Hzzm28VsXEJaQk5EAGMMPiAC9/" +
			"wgBNb3gdHk9Sorz0+dgEwEVvQexToGj8SsrGyKhsgCROBUw0ko27GezQwfQ8YJaMAoChijoKiC1HN4jXLymhOoLIIXST6PUr+TkA5VnHXpRXB4/DTIzy8AjzuVCAg" +
			"QDAbhzNmzUF93CKZs+xRGk2JUNR4FLiMqRfjYsFr28jUl8F4BiCSfetJ+noW8LknA9lnTcNOtsyC3Vy/mzcpGp9PFb84inZ3Y3NoCDQ0NrPVAHc7bVQE9BZlRFOKV" +
			"irBpWC2Mu6YESD43k3y+IuPRRJ5vFSRWccvdmFraDzKzspkrJRVtZlOcQEyW0R+KQMvZRkYkUN35LTx8rJo5SXJUTiGWyO2C4Qfg9LUk8Ap5f4GFbFQpAqtLRrItA" +
			"aSmzQEFPbxs/PTp+Mmbb4HZZmcOM2L5T2fBJ0ueZ02+IIb8frjDxtgs/yHERAR4VXqECCy+JgTeK4zL5wTtsjlUJWFz72HQOnIi7NmxH2wOO2SnumDijOnwEREQzD" +
			"Zw2yQYf9dsWLn4RQiEYxDwtULB4CEwZtNqGBc9G3c/VaONRGDCNSGwrlS8mTaujTYzsLqMPDzwswchLyuVbVlfhYLZAi6bmd1590z88PXXqDRZmcdhwdvvvhdW/P0" +
			"lFo7KKIeDUDRmHGuqq8OfV30EfbQIJ6CQlPJH1sGZq05gTbHwilPUfu2zWVn1tIcwt99AyPS42IbVn6IiSGAVgf1k9mxc9caroAiWOIHJ98yFFUteotKJKGgylE2a" +
			"TPlwAkM7d8DcfWvBSV1GhwoLxh2EpVeVwLO5IA6xQwNK4P2YpGMvGwQ98rzgIukc3F4NUb7FUs8wdNwE+PbLNaCiCHarGQaNnQBb135OO52J7w3gLR0AzS0+OHmyC" +
			"TJq98CcztMQ1qDyy6Bw66LTfK++CgQyJcCX82Bcpgkq16Xlob9sAMsvyEW3xw2i3cOObq9Gs0ANpiqzvMHl2Lz3a2rakJltNswsGQTHv60mRdniz0vvO4Q1t0ewrf" +
			"ksNJ1qhPK679gtEFJqI1D4wAloitJOTcu6RKQrBPgaLDKD8KQXXjoiOh7amJGLGVlpzJOWiia7mzxtZQw1tGKM2mjGFI3RQIO876FGmlEHSkMCrzmCQDHRQFNk1ha" +
			"Ioa+9AzrafaD429i8yBk8EVEW/K0J3ogwWgRxXJLEpQigDpF6HvOfc6By083TYjeMKR+d4nKBEg5DJMJ7TBG4YTaqOK6sTODtc9PRE/QfBIn664zCHsAJ+U6eBjne" +
			"V2sQU1RwOK2gkawC1PDVfLrm32MObtcebwQ+dqo6WBK6TeC/xnPkSFDkECAfS/u7Hv3T4/wFFTu2dSPW7D9OPhYYMhVzslLhxmkz6FzDVYuXgCyYWAol8R1z5xJHE" +
			"9vw5lIMKPx2tMGpDEeNHAxZfctAoJ7pdzMfHDFE67hxSwiWk+ujAPGBR71UJC5GQNB/p70WqOJTjwzgTMvMKHqtYvkavuDI1g2455vdYLNIVF8A3W4XjJ05jx7GsO" +
			"WzF8Cb6WYN7Qxzb59PQ7LIPvzrU6iINj5zsqis4qQpkyG77w0QCYaCsydMnUQTTxvdtoMQgf8lccEoXIgAJhlvIli58YS0gnTLwGeWvfSOM7eYRkbqB5iaWIx0zic" +
			"ZyRa/snXtC5Cd7oRTfgG8k+5P3FQJc26UG3yiUUATTEBEIFBbFZ5//1P3kLwaaZlPJ0GLIXZOFFh3CIg6LAQHwUPImnVTyqzhsxY84C4exlKcLjRTz8P7foHcKgiU" +
			"CzQP8OvPVDxHJVRiIXShd+J9oCoq00hadMTPY3IMo51RaG9vB3awkr3z+gfPbDsSrqJrmwhGJHgU5ItJCS9gvEFA0r2fQkgn5D57V+YfpeFzR4jevsxus6PJZIprm" +
			"ltGJCDxELIy2sFP6I8mZLMdyHo6+HzMIHHKkEchFAoCHqtiR7dVfL5kne89up7PBs0Evx6F6MWicCkChvbd3PtkSsGGJwqXhQbMdThcHvK8Ob6UGxWx54NsTtVvz+" +
			"KS4jzi32gQ1rTEs62xM2COnIV4J0fXRWNRCDcfA9+u1cfnLD31F1pUTzijSylI6LxYLlyIgKAT4O616/LxEgqfnz/8hazyqRnU75NsTPHrrVobS89JR39KOZhEiYZ" +
			"6IS4V/jA6aIzUuOaZLCuYF/gc6lq9NMGptIJyQY6CP+Bn+1YvO/TcmsZX6ZpjhFOEFkJAJxDTI6B1l4ARAXIt5BB6L3p44pNFA8p6iTQDKJqGnZ0yWJiP5ZQMRCy8" +
			"DRzUQktU/AU05ARxArFYjLUHAuhu+AB218nM4bCihfpx/g5JURVW+cVX+15fvXsFrecvvRoIZ3UZRS6WBxcjIOgE7DqBfMKAX92Wf69fsdhIx2ZVYxZZ0awjiy2Ok" +
			"fmSFRSEFKdE46UQ37gSIaAnq5yABv6gDKnUk7xaGYy1hZlMI6dMhUs2iYLSGWw/verr5i/oklrCUT2ZDQLdjoCRA6akCPQhlM8YYp/+r92RABHgHuEVyjX/9szcUT" +
			"mYEuqIYZpLFCI0pfBeAvWnabT70vTGfB0q613swSc+bPLvre8M6fLQvCmiVJQp+auORHkV2kk4SGjUCXTqEegyAUiKgFGFPDqBm27IM0840CRjTGEhnaT7tzNy+/Q" +
			"xxzyxqIIZZExYJ6AnAW9QwU7p0hJQ1fyeKfDyOp9vc02QJymvMEIPj2RTGWtp9Kvf0PfthDo9kY1SqkA3qtD58sBFyCX0J/SFRFkFXV5ZC+cUDk4JdnhMIhPTnKIp" +
			"yAnwAOgh4BXIQa4gAkpGthOW7wg1VWxr4zoPwvfVpZWwn7CXcFz/HoLL2AeSZSTA9zsxL6XZkEjmVJ1EFs+NxY/0GZ8X9WfYbYJot5Kw+U4gQJxBPAIaDWdUjTrCq" +
			"mp12fD9XZETb3/RvF/3sl8nwg0+AYkEbu6K9y9G4NwoGCQcOhEuqUxIJHbp+4+XTO0V9Wc6HKJgsQpIkxfo/o8nMd/EJNrrwhHKZhqkl9eqR59beapa9/Rp3Xifjn" +
			"boQv3vCgEjF5ITmietTfd+BqEnYfDCaanTM2yYarMKkkVCsVNNlE+eBSw+nDDmNKHQFqJ6SXvatgal/q2qwJeQqDjc4y16JLhkjB7IkM5lt9PnSsloLYzSyqNQSBj" +
			"6y5scUzJcols0CaBvYIKxC9PVJCA+AmjcDIHGNGHXKbl+/f7Ievq1BhKbVrtuvLFpGbK55FDT5YnsPCQcuoyKCUW6tJJLsFFFjQQE/VpBlwyvNMd07xvGG16/pOe7" +
			"QyCZRHKTZ9JJpEGi0XPo/zd+F/Rrkz2J+jlPUJ6oPt348w0wV2wmPne9sUcYhlqSICWRMAgYxhhEVN1gY2iR4f8rTZffTFzOq8XkaBhEjE0v+XvyvZNJqOcgOULde" +
			"qVyuQTORySZEF7gvsnePdfb3Tb8ShA43z0uZHwyifN9XpGHX+njQlG4osd/AAHSeXwT8bWrAAAAAElFTkSuQmCC";

var checkForInitialize = function() {
	if(!ss.storage.vertical_location) {
		ss.storage.vertical_location = "bottom";
	}
	if(!ss.storage.horizontal_location) {
		ss.storage.horizontal_location = "right";
	}
	if(!ss.storage.scrolling_speed && 0 != ss.storage.scrolling_speed) {
		ss.storage.scrolling_speed = "1200";
	}
	if(!ss.storage.visibility_behavior) {
		ss.storage.visibility_behavior = "alwaysshow";
	}
	if (!ss.storage.icon_transparency) {
		ss.storage.icon_transparency = "0.5";
	}
	if (!ss.storage.black_and_white) {
		ss.storage.black_and_white = "false";
	}
	
	if (!ss.storage.arrow_type) {
		ss.storage.arrow_type = "1";
	}
	
	if (!ss.storage.smart_direction_mode) {
		ss.storage.smart_direction_mode = "false";
	}
	if(!ss.storage.control_options) {
		ss.storage.control_options = "pager";
	}
	if (!ss.storage.hide_controls) {
		ss.storage.hide_controls = "true";
	}
	if(!ss.storage.icon_size) {
		ss.storage.icon_size = "48";
	}
	if(!ss.storage.icon_library) {
		ss.storage.icon_library = "1";
	}
	if(!ss.storage.user_saved_icon) {
		ss.storage.user_saved_icon = DEFAULT_ICON_BASE_64;
	}
	
	if(!ss.storage.d_arrangement) {
		ss.storage.d_arrangement = "hr";
	}
	if (!ss.storage.d_icon_library) {
		ss.storage.d_icon_library = "1";
	}
	if (!ss.storage.d_user_saved_icon) {
		ss.storage.d_user_saved_icon = DEFAULT_ICON_BASE_64;
	}
	
	if (!ss.storage.h_offset) {
		ss.storage.h_offset = "20";
	}
	if (!ss.storage.v_offset) {
		ss.storage.v_offset = "20";
	}
	if (!ss.storage.removed_sites) {
		ss.storage.removed_sites = "mail.google.com/mail;google.com/calendar;";
	}
};

// initialize settings
checkForInitialize();

var forceInitializeSettings = function() {
	delete ss.storage.vertical_location;
	delete ss.storage.horizontal_location;
	delete ss.storage.scrolling_speed;
	delete ss.storage.visibility_behavior;
	delete ss.storage.icon_transparency;
	delete ss.storage.black_and_white;
	
	delete ss.storage.arrow_type;
	
	delete ss.storage.smart_direction_mode;
	delete ss.storage.control_options;
	delete ss.storage.hide_controls;
	delete ss.storage.icon_size;
	delete ss.storage.icon_library;
	delete ss.storage.user_saved_icon;
	
	delete ss.storage.d_icon_library;
	delete ss.storage.d_user_saved_icon;
	delete ss.storage.d_arrangement;
	
	delete ss.storage.h_offset;
	delete ss.storage.v_offset;
	delete ss.storage.removed_sites;
	
	checkForInitialize();
};

// Create a page mod
// The script replaces the page contents with a message
pageMod.PageMod({
    include: "*", // for all pages, as it will give the scrolling for all pages
	attachTo: ["existing", "top"],
    contentStyleFile: data.url("pratikabu-stt.css"),
    contentScriptFile: [data.url("thirdparty/pratikabu-jquery.js"), data.url("thirdparty/pratikabu-jquery-rotate.js"),
        data.url("browserspecific/pratikabu-stt-impl.js"), data.url("pratikabu-stt.js")],
	contentScriptWhen: "ready",
    onAttach: function onAttach(worker) {// attaching the worker so as to do the communication with contentscript file
		worker.port.on('getPrefs', function() {
			var dataJson = {
				vLoc: ss.storage.vertical_location,
				hLoc: ss.storage.horizontal_location,
				scrSpeed: ss.storage.scrolling_speed,
				visibilityBehav: ss.storage.visibility_behavior,
				iconTransparency: ss.storage.icon_transparency,
				blackAndWhite: ss.storage.black_and_white,
				
				arrowType: ss.storage.arrow_type,
				
				iconSize: ss.storage.icon_size,
				
				hOffset: ss.storage.h_offset,
				vOffset: ss.storage.v_offset,
				removedSites: ss.storage.removed_sites
			};
			
			// bypass settings if they are not required
			if("1" == dataJson.arrowType) {// send only single arrow settings
				dataJson.smartDirection = ss.storage.smart_direction_mode;
				dataJson.controlOption = ss.storage.control_options;
				dataJson.hideControls = ss.storage.hide_controls;
				dataJson.iconLib = ss.storage.icon_library;
				if("myIcon" == dataJson.iconLib) {
					dataJson.userIcon = ss.storage.user_saved_icon;
				}
			} else {// send only dual arrow settings
				dataJson.dArrang = ss.storage.d_arrangement;
				dataJson.dIconLib = ss.storage.d_icon_library;
				if("myIcon" == dataJson.dIconLib) {
					dataJson.dUserIcon = ss.storage.d_user_saved_icon;
				}
			}
			
			dataJson.iconFolderLocation = data.url("icons");
			
			worker.port.emit("prefsValue", dataJson);
		});
		
		worker.port.on('optionPage', function() {
			openOptioinPage();
		});
    }
});

var openOptioinPage = function(updated) {
	// don't open the option page if it is already open
	var optionPageUrl = data.url("options/options.html");
	var index;
	for (index = 0; index < tabs.length; ++index) {
		var openTab = tabs[index];
		if(openTab.url.toLowerCase().indexOf(optionPageUrl.toLowerCase()) != -1) {
			// page is already open
			openTab.activate();
			return;
		}
	}
	
	if(updated) {
		updated = "?updated=true";
	} else {
		updated = "";
	}
    tabs.open({
		url: data.url("options/options.html" + updated),
		onReady: runScript
	});
	function runScript(tab) {
		tabWorker = tab.attach({
			contentScriptFile: [data.url("thirdparty/pratikabu-jquery.js"), data.url("thirdparty/pratikabu-simple-slider.js"), data.url("browserspecific/options-browser-impl.js"), data.url("options/res/options.js")]
		});
		
		tabWorker.port.on('getPrefs', function() {
			tabWorker.port.emit("prefsValue", {// send the image url, bottom preference and the left preference
				vLoc: ss.storage.vertical_location,
				hLoc: ss.storage.horizontal_location,
				scrSpeed: ss.storage.scrolling_speed,
				visibilityBehav: ss.storage.visibility_behavior,
				iconTransparency: ss.storage.icon_transparency,
				blackAndWhite: ss.storage.black_and_white,
				
				arrowType: ss.storage.arrow_type,
				
				smartDirection: ss.storage.smart_direction_mode,
				controlOption: ss.storage.control_options,
				hideControls: ss.storage.hide_controls,
				iconSize: ss.storage.icon_size,
				iconLib: ss.storage.icon_library,
				userIcon: ss.storage.user_saved_icon,
				
				dArrang: ss.storage.d_arrangement,
				dUserIcon: ss.storage.d_user_saved_icon,
				dIconLib: ss.storage.d_icon_library,
				
				hOffset: ss.storage.h_offset,
				vOffset: ss.storage.v_offset,
				removedSites: ss.storage.removed_sites,
				
				supportPrompt: ss.storage.support_prompt
			});
		});
		
		tabWorker.port.on('setPrefs', function(data) {
			ss.storage.vertical_location = data.vLoc;
			ss.storage.horizontal_location = data.hLoc;
			ss.storage.scrolling_speed = data.scrSpeed;
			ss.storage.visibility_behavior = data.visibilityBehav;
			ss.storage.icon_transparency = data.iconTransparency;
			ss.storage.black_and_white = data.blackAndWhite;
			
			ss.storage.arrow_type = data.arrowType;
			
			ss.storage.smart_direction_mode = data.smartDirection;
			ss.storage.control_options = data.controlOption;
			ss.storage.hide_controls = data.hideControls;
			ss.storage.icon_size = data.iconSize;
			ss.storage.icon_library = data.iconLib;
			ss.storage.user_saved_icon = data.userIcon;
			
			ss.storage.d_user_saved_icon = data.dUserIcon;
			ss.storage.d_icon_library = data.dIconLib;
			ss.storage.d_arrangement = data.dArrang;
			
			ss.storage.h_offset = data.hOffset;
			ss.storage.v_offset = data.vOffset;
			ss.storage.removed_sites = data.removedSites;
			
			ss.storage.support_prompt = data.supportPrompt;
			
			tabWorker.port.emit("saveStatus", {// send the status of the step
				status: "Saved perfectly."
			});
		});
		
		tabWorker.port.on('resetPrefs', function() {
			forceInitializeSettings();
			
			tabWorker.port.emit("resetStatus", {// send the status of the step
				status: "Saved perfectly."
			});
		});
		
		tabWorker.port.emit("loadJavascriptFunctions");
	};
};

sp.on("myButtonPref", openOptioinPage);

// open option page on initial in this version
var currentVersion = 9.1;// this variable should be incremented with every update so that, add-on update message can be shown
if(!ss.storage.versionInfo || currentVersion > ss.storage.versionInfo) {
	ss.storage.versionInfo = currentVersion;
	openOptioinPage("updated");
}